package com.muyu.rule.server.controller;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.DataValue;
import com.muyu.rule.common.domain.RuleEngineVersion;
import com.muyu.rule.common.domain.req.VersionAddReq;
import com.muyu.rule.common.utils.LoadEngineClass;
import com.muyu.rule.server.service.RuleEngineVersionService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @Author：张承志
 * @Package：com.muyu.rule.server.controller
 * @Project：cloud-etl-rule
 * @name：RuleEngineVersionController
 * @Date：2024/8/25 10:41
 */
@Log4j2
@RestController
@RequestMapping("/version")
@RequiredArgsConstructor
@Tag(name = "规则版本控制层", description = "规则版本管理,查看等相关操作")
public class RuleEngineVersionController {

    /**
     * 保存代码
     * @param ruleEngineVersion
     * @return
     */
    @PostMapping("/saveCode")
    @Operation(summary = "通过id回显规则版本",description = "传入规则版本的主键id,返回规则版本的对象")
    public Result saveCode(@RequestBody RuleEngineVersion ruleEngineVersion){

        log.info("==>"+ruleEngineVersion.getVersionClazz());
        UpdateWrapper<RuleEngineVersion> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id",ruleEngineVersion.getId()).set("version_clazz",ruleEngineVersion.getVersionClazz());
         versionService.Upload(ruleEngineVersion.getVersionClazz(),ruleEngineVersion.getClassName());
        versionService.update(updateWrapper);
        return   Result.success();
    }

    @PostMapping("/findVersionById/{id}")
    @Operation(summary = "通过id回显规则版本",description = "传入规则版本的主键id,返回规则版本的对象")
    public Result<RuleEngineVersion>findVersionById(@PathVariable("id") Long id){

        RuleEngineVersion ruleEngineVersion = versionService.getById(id);

        return Result.success(ruleEngineVersion);
    }



    @Autowired
    private RuleEngineVersionService versionService;

    /**
     * 添加规则引擎版本
     * @param versionAddReq 引擎版本添加实体类
     * @return
     */
 @PostMapping("/insertVersion")
 @Operation(summary = "增加规则引擎版本",description = "通过传入版本添加的实体类,进行规则引擎的版本添加")
    public Result insertVersion(@Validated @RequestBody VersionAddReq versionAddReq){

     boolean save = versionService.save(RuleEngineVersion.addBuild(versionAddReq));
     if (true==save){
      //添加成功则把增加的代码进行java编译,进行热加载
         versionService.complierClass(versionAddReq.getClassName(),versionAddReq.getVersionClazz());
     }

     return  Result.success();
 }

    /**
     * 激活引擎请求
     * @param id
     * @return
     */
    @PostMapping("/activate/{id}")
    @Operation(summary = "规则引擎的激活", description = "通过Id激活规则引擎")
    public Result activate(@PathVariable("id") Long id){

        versionService.activate(id);

        return Result.success();
    }

    /**
     * 取消引擎请求
     */

    @PostMapping("/disable/{id}")
    @Operation(summary = "规则引擎的取消", description = "通过Id取消规则引擎")
    public Result disable(@PathVariable("id") Long id){

        versionService.disable(id);

        return Result.success();
    }



    /**
     * 生成代码
     * @param
     * @return
     */

    @PostMapping("/deposit/{type}")
    public Result deposit(@PathVariable("type") Integer type,
                          @Validated @RequestBody VersionAddReq versionAddReq) {

        log.info("传入参数{}", type);

        // 根据类型确定父类
        String parentClass = determineParentClass(type);

        // 定义模板字符串
        String template = "package com.muyu.rule.common.engine;\n" +
                "\n" +
                "import cn.hutool.core.util.DesensitizedUtil;\n" +
                "import com.alibaba.fastjson2.JSON;\n" +
                "import com.alibaba.fastjson2.JSONObject;\n" +
                "import com.muyu.common.domain.DataValue;\n" +
                "import com.muyu.rule.common.basic.abstracts." + parentClass + ";\n" +
                "\n" +
                "\n" +
                "/**\n" +
                " * @Author：张承志\n" +
                " * @Package：com.muyu.rule.server.basic.engine.row\n" +
                " * @Project：cloud-etl-rule\n" +
                " * @name：数据集指定字段进行脱敏\n" +
                " * @Date：2024/8/30 11:13\n" +
                " */\n" +
                "\n" +
                "public class ${className} extends " + parentClass + " {\n" +
                "\n" +
                "    @Override\n" +
                "    public void run() {\n" +
                "\n" +
                "        DataValue[][] dataValues = get();\n" +
                "\n" +
                "        String dataValue = getDataValue();\n" +
                "\n" +
                "        JSONObject jsonObject = JSON.parseObject(dataValue);\n" +
                "\n" +
                "        String key = (String)jsonObject.get(\"key\");\n" +
                "\n" +
                "        for (DataValue[] value : dataValues) {\n" +
                "            for (DataValue dataValue1 : value) {\n" +
                "                if (dataValue1.getKey().equals(key)) {\n" +
                "                    dataValue1.setValue(DesensitizedUtil.mobilePhone((String) dataValue1.getValue()));\n" +
                "                }\n" +
                "            }\n" +
                "        }\n" +
                "        set(dataValues);\n" +
                "\n" +
                "    }\n" +
                "}\n";

        // 使用Map进行模板替换
        Map<String, String> replacements = Map.of(
                "${className}", versionAddReq.getClassName(),
                "${parentClass}", parentClass
        );

        // 替换模板中的变量
        String versionClass = replacePlaceholders(template, replacements);

        versionAddReq.setVersionClazz(versionClass);

        return Result.success(versionAddReq);
    }

    // 辅助方法用于根据类型决定父类
    private String determineParentClass(Integer type) {
        switch (type) {
            case 1:
                return "DataEngineValueActuator";
            case 2:
                return "DataEngineRowActuator";
            case 3:
                return "DataEngineDataSetActuator";
            default:
                throw new IllegalArgumentException("Invalid type provided");
        }
    }

    // 辅助方法用于替换模板中的占位符
    private String replacePlaceholders(String template, Map<String, String> replacements) {
        for (Map.Entry<String, String> entry : replacements.entrySet()) {
            template = template.replace(entry.getKey(), entry.getValue());
        }
        return template;
    }


//    @PostMapping("/deposit/{type}")
//    public Result deposit(@PathVariable("type")Integer type,@Validated @RequestBody VersionAddReq versionAddReq){
//
//        log.info("传入参数{}"+type);
//
//        //String versionClass = versionService.deposit(type);
//
//        String parentClass = null;
//        if (type==1){
//            parentClass = "DataEngineValueActuator";
//        }else if (type==2){
//            parentClass="DataEngineRowActuator";
//        }else if (type==3){
//            parentClass="DataEngineDataSetActuator";
//        }
//
//        String template = "package com.muyu.rule.common.engine;\n" +
//                "\n" +
//                "import cn.hutool.core.util.DesensitizedUtil;\n" +
//                "import com.alibaba.fastjson2.JSON;\n" +
//                "import com.alibaba.fastjson2.JSONObject;\n" +
//                "import com.muyu.common.domain.DataValue;\n" +
//                "import com.muyu.rule.common.basic.abstracts.${parentClass};\n" +
//                "\n" +
//                "\n" +
//                "/**\n" +
//                " * @Author：张承志\n" +
//                " * @Package：com.muyu.rule.server.basic.engine.row\n" +
//                " * @Project：cloud-etl-rule\n" +
//                " * @name：数据集指定字段进行脱敏\n" +
//                " * @Date：2024/8/30 11:13\n" +
//                " */\n" +
//                "\n" +
//                "public class ${className} extends ${parentClass} {\n" +
//                "\n" +
//                "    @Override\n" +
//                "    public void run() {\n" +
//                "\n" +
//                "        DataValue[][] dataValues = get();\n" +
//                "\n" +
//                "        String dataValue = getDataValue();\n" +
//                "\n" +
//                "        JSONObject jsonObject = JSON.parseObject(dataValue);\n" +
//                "\n" +
//                "        String key = (String)jsonObject.get(\"key\");\n" +
//                "\n" +
//                "        for (DataValue[] value : dataValues) {\n" +
//                "            for (DataValue dataValue1 : value) {\n" +
//                "                if (dataValue1.getKey().equals(key)){\n" +
//                "                    dataValue1.setValue(DesensitizedUtil.mobilePhone((String) dataValue1.getValue()));\n" +
//                "                }\n" +
//                "            }\n" +
//                "        }\n" +
//                "        set(dataValues);\n" +
//                "\n" +
//                "    }\n" +
//                "}\n";
//
//           String versionClass = template.replace("${className}", versionAddReq.getClassName());
//           versionClass = template.replace("${parentClass}",parentClass);
//
//           versionAddReq.setVersionClazz(versionClass);
//
//
//        return Result.success(versionAddReq);
//    }

@PostMapping("/readByOss/{className}")
@Operation(summary = "从Oss获取数据", description = "传入参数oss的key名从oss读取数据")
    public Result readByOss(@PathVariable String className){

Object  o=  versionService.readByOss(className);

    return Result.success(o);
    }




    /**
     * 通过id查询对应的版本
     * @param id
     * @return
     */

@PostMapping("/selectVersionById/{id}")
public Result<List<RuleEngineVersion>> selectVersionById(@PathVariable("id") Long id){

    List<RuleEngineVersion> versionList = versionService.selectRuleEngineVersion(id);

    return Result.success(versionList);
}




    @Autowired
    private LoadEngineClass loadEngineClass;

    @PostMapping("/testEngine/{className}")
    @Operation(summary = "字段规则引擎测试", description = "传入规则引擎的类名,和数据,用规则引擎进行测试数据")
    public Result testEngine(@PathVariable("className") String className,@RequestBody DataValue dataValue){
        log.info("数据:" + dataValue);
        //加载对应的规则引擎
        loadEngineClass.loadValueEngineClass(className);
        //执行对应的规则引擎
        DataValue value = versionService.testEngine(className, dataValue);

        return Result.success(value);
    }

    @PostMapping("/rowEngine/{className}")
    @Operation(summary = "行/记录规则引擎测试", description = "传入规则引擎的类名,和数据,用规则引擎进行测试数据")
    public Result testEngine(@PathVariable("className") String className,@RequestBody DataValue[] dataValue){
        log.info("数据:" + dataValue);
        //加载对应的规则引擎
        loadEngineClass.loadRowEngineClass(className);
        //执行对应的规则引擎
        DataValue[] value = versionService.testEngine(className, dataValue);

        return Result.success(value);
    }
@PostMapping("/DataSetEngine/{className}")
@Operation(summary = "数据集的规则引擎测试", description = "传入规则引擎的类名,和数据,用规则引擎进行测试数据")
public Result DataSetEngine(@PathVariable("className") String className,@RequestBody DataValue[][] dataValues){

    loadEngineClass.loadDataSetEngineClass(className);

    DataValue[][] testedEngine = versionService.testEngine(className, dataValues);

    return Result.success(testedEngine);
}










}



